==> Botan-1.1.1.tgz: Botan-1.1.1/src/rsa.cpp <==
/*************************************************
* RSA Source File                                *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/rsa.h>
#include <botan/numthry.h>
#include <botan/asn1.h>

namespace Botan {

/*************************************************
* RSA_PublicKey Constructor                      *
*************************************************/
RSA_PublicKey::RSA_PublicKey(const BigInt& mod, const BigInt& exp)
   {
   if_initialize(mod, exp, true);
   if(!check_params())
      throw Invalid_Argument("RSA: invalid paramaters for public key");
   }

/*************************************************
* RSA_PublicKey Constructor                      *
*************************************************/
RSA_PublicKey::RSA_PublicKey(DataSource& source, X509_Encoding encoding)
   {
   X509_decode(source, encoding);
   }

/*************************************************
* RSA Encryption Function                        *
*************************************************/
SecureVector<byte> RSA_PublicKey::encrypt(const byte in[], u32bit len) const
   {
   BigInt temp(in, len);
   return encode(public_op(temp));
   }

/*************************************************
* RSA Verification Function                      *
*************************************************/
SecureVector<byte> RSA_PublicKey::verify(const byte in[], u32bit len) const
   {
   BigInt temp(in, len);
   return encode(public_op(temp));
   }

/*************************************************
* RSA Public Operation                           *
*************************************************/
BigInt RSA_PublicKey::public_op(const BigInt& i) const
   {
   if(i >= n || i.is_negative())
      throw Invalid_Argument("RSA::public_op: i >= n || i < 0");
   return powermod_e_n(i);
   }

/*************************************************
* Return the X.509 public key encoding           *
*************************************************/
SecureVector<byte> RSA_PublicKey::encode_key_x509() const
   {
   DER_Encoder encoder;
   encoder.start_sequence();
   DER::encode(encoder, n);
   DER::encode(encoder, e);
   encoder.end_sequence();
   return encoder.get_contents();
   }

/*************************************************
* Decode a X.509 public key encoding             *
*************************************************/
void RSA_PublicKey::decode_key_x509(DataSource& source)
   {
   BER_Decoder decoder(source);
   BER_Decoder sequence = BER::get_subsequence(decoder);
   BigInt mod, exp;
   BER::decode(sequence, mod);
   BER::decode(sequence, exp);
   sequence.verify_end();
   if_initialize(mod, exp, true);
   if(!check_params())
      throw Decoding_Error("RSA: invalid X.509 public key");
   }

/*************************************************
* Return the algorithm OID                       *
*************************************************/
OID RSA_PublicKey::get_oid() const
   {
   return OID("1.2.840.113549.1.1.1");
   }

/*************************************************
* RSA_PrivateKey Constructor                     *
*************************************************/
RSA_PrivateKey::RSA_PrivateKey(const BigInt& prime1, const BigInt& prime2,
                               const BigInt& exp, const BigInt& d_exp,
                               const BigInt& modulus) :
   RSA_PublicKey(((!modulus) ? prime1 * prime2 : modulus), exp)
   {
   if(prime1 < 3 || prime2 < 3)
      throw Invalid_Argument("RSA_PrivateKey: invalid prime(s)");
   if(d_exp != 0 && d_exp < 3)
      throw Invalid_Argument("RSA_PrivateKey: invalid decryption exponent");

   p = prime1;
   q = prime2;
   d = (!d_exp) ? inverse_mod(e, lcm(p - 1, q - 1)) : d_exp;
   precompute();
   }

/*************************************************
* Create a RSA Key                               *
*************************************************/
RSA_PrivateKey::RSA_PrivateKey(u32bit bits, const BigInt& exp)
   {
   if(bits < 512)
      throw Invalid_Argument("RSA: Can't make a key that is only " +
                             to_string(bits) + " bits long");
   if(exp < 3 || exp % 2 == 0)
      throw Invalid_Argument("RSA: Invalid encryption exponent");

   e = exp;
   p = random_prime((bits + 1) / 2, LongTermKey, e);
   q = random_prime(bits - p.bits(), LongTermKey, e);
   n = p * q;
   d = inverse_mod(e, lcm(p - 1, q - 1));
   precompute();
   }

/*************************************************
* RSA Decryption Operation                       *
*************************************************/
SecureVector<byte> RSA_PrivateKey::decrypt(const byte in[], u32bit len) const
   {
   BigInt temp(in, len);
   return encode(if_private_op(temp));
   }

/*************************************************
* RSA Signature Operation                        *
*************************************************/
SecureVector<byte> RSA_PrivateKey::sign(const byte in[], u32bit len) const
   {
   BigInt temp(in, len);
   return encode(if_private_op(temp));
   }

/*************************************************
* Check Private RSA Parameters                   *
*************************************************/
bool RSA_PrivateKey::check_params() const
   {
   if(!IF_Scheme_PrivateKey::check_params())
      return false;
   if((e * d) % lcm(p - 1, q - 1) != 1)
      return false;
   return true;
   }

}

==> Botan-1.1.2.tgz: Botan-1.1.2/src/rsa.cpp <==
/*************************************************
* RSA Source File                                *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/rsa.h>
#include <botan/numthry.h>
#include <botan/asn1.h>

namespace Botan {

/*************************************************
* RSA_PublicKey Constructor                      *
*************************************************/
RSA_PublicKey::RSA_PublicKey(const BigInt& mod, const BigInt& exp)
   {
   if_initialize(mod, exp, true);
   if(!check_params())
      throw Invalid_Argument("RSA: invalid paramaters for public key");
   }

/*************************************************
* Decode a X.509 RSA public key                  *
*************************************************/
RSA_PublicKey::RSA_PublicKey(DataSource& source, X509_Encoding encoding)
   {
   X509::decode(*this, source, encoding);
   }

/*************************************************
* RSA Encryption Function                        *
*************************************************/
SecureVector<byte> RSA_PublicKey::encrypt(const byte in[], u32bit len) const
   {
   BigInt temp(in, len);
   return encode(public_op(temp));
   }

/*************************************************
* RSA Verification Function                      *
*************************************************/
SecureVector<byte> RSA_PublicKey::verify(const byte in[], u32bit len) const
   {
   BigInt temp(in, len);
   return encode(public_op(temp));
   }

/*************************************************
* RSA Public Operation                           *
*************************************************/
BigInt RSA_PublicKey::public_op(const BigInt& i) const
   {
   if(i >= n || i.is_negative())
      throw Invalid_Argument("RSA::public_op: i >= n || i < 0");
   return powermod_e_n(i);
   }

/*************************************************
* Return the algorithm OID                       *
*************************************************/
OID RSA_PublicKey::get_oid() const
   {
   return OID("1.2.840.113549.1.1.1");
   }

/*************************************************
* RSA_PrivateKey Constructor                     *
*************************************************/
RSA_PrivateKey::RSA_PrivateKey(const BigInt& prime1, const BigInt& prime2,
                               const BigInt& exp, const BigInt& d_exp,
                               const BigInt& modulus) :
   RSA_PublicKey(((!modulus) ? prime1 * prime2 : modulus), exp)
   {
   if(prime1 < 3 || prime2 < 3)
      throw Invalid_Argument("RSA_PrivateKey: invalid prime(s)");
   if(d_exp != 0 && d_exp < 3)
      throw Invalid_Argument("RSA_PrivateKey: invalid decryption exponent");

   p = prime1;
   q = prime2;
   if(p < q) std::swap(p, q);
   d = (!d_exp) ? inverse_mod(e, lcm(p - 1, q - 1)) : d_exp;
   precompute();
   }

/*************************************************
* Create a RSA private key                       *
*************************************************/
RSA_PrivateKey::RSA_PrivateKey(u32bit bits, const BigInt& exp)
   {
   if(bits < 512)
      throw Invalid_Argument("RSA: Can't make a key that is only " +
                             to_string(bits) + " bits long");
   if(exp < 3 || exp % 2 == 0)
      throw Invalid_Argument("RSA: Invalid encryption exponent");

   e = exp;
   p = random_prime((bits + 1) / 2, LongTermKey, e);
   q = random_prime(bits - p.bits(), LongTermKey, e);
   if(p < q) std::swap(p, q);
   n = p * q;
   d = inverse_mod(e, lcm(p - 1, q - 1));
   precompute();
   }

/*************************************************
* Decrypt/decode a PKCS #8 RSA private key       *
*************************************************/
RSA_PrivateKey::RSA_PrivateKey(DataSource& source, const std::string& pass,
                               X509_Encoding encoding)
   {
   PKCS8::decrypt_key(*this, source, pass, encoding);
   }

/*************************************************
* RSA Decryption Operation                       *
*************************************************/
SecureVector<byte> RSA_PrivateKey::decrypt(const byte in[], u32bit len) const
   {
   BigInt temp(in, len);
   return encode(if_private_op(temp));
   }

/*************************************************
* RSA Signature Operation                        *
*************************************************/
SecureVector<byte> RSA_PrivateKey::sign(const byte in[], u32bit len) const
   {
   BigInt temp(in, len);
   return encode(if_private_op(temp));
   }

/*************************************************
* Check Private RSA Parameters                   *
*************************************************/
bool RSA_PrivateKey::check_params() const
   {
   if(!IF_Scheme_PrivateKey::check_params())
      return false;
   if((e * d) % lcm(p - 1, q - 1) != 1)
      return false;
   return true;
   }

}
